#!/usr/bin/python
# -*- coding: utf-8 -*-

# download.py
#
# This file is part of BotMU
#
# Author: Raphaël Perraudeau <r.perraudeau@gmail.com>
# Copyright: 2011 Raphaël Perraudeau <r.perraudeau@gmail.com>
# Licence: GNU/GPL
#
# Home page: <http://botmu.sourceforge.net/>


import os
import sys
import datetime
import re
from htmlentitydefs import name2codepoint as n2cp

from PyQt4.QtCore import *

from commonFunctions import *
from httpReq import *


#############################################################################
class Abort(Exception):
	"""classe d'exception crée pour l'arrêt du téléchargement avant la fin"""
	pass
 
#############################################################################
class TelechargerMegaupload(QThread):
	"""Thread de téléchargement"""

	#========================================================================
	def __init__(self, fileList, destination, parent=None):
		super(TelechargerMegaupload,self).__init__()
		self.destination = destination
		self.stop = False
		self.httpreq = httpReq(parent.configDir)
		self.fileRunning = ""
		self.downloadingFile = 0
		self.parent = parent
		self.fileList = fileList
		
		setupFile = os.path.join(self.parent.configDir, "setup.ini")
		paramfile = open(setupFile)
		param = (paramfile.read()).split("\n")
		for item in param:
			if item.split("=")[0] == 'proxy':
				self.proxy = item.split("=")[1]
				self.httpreq.defineProxy(self.proxy)
			if item.split("=")[0] == 'useragent':
				self.useragent = item.split("=")[1]
				self.httpreq.defineUseragent(self.useragent)
			if item.split("=")[0] == 'verbose':
				self.verbose = item.split("=")[1]
				self.httpreq.defineVerbose(int(self.verbose))
			if item.split("=")[0] == 'limit':
				self.limit = item.split("=")[1]
				self.httpreq.defineLimit(int(self.limit))

	#========================================================================
	def run(self):
		row = 0
		self.stop = False
		while self.fileList:
			if len(self.fileList)>row:
				MUSrc = self.fileList[row][0]
				dest = self.fileList[row][1]
				continuedl = self.fileList[row][3]
				if(continuedl != 100):
					self.downloadingFile = row
					self.downloadFile(MUSrc, dest, row, continuedl)
			else:
				break
			row+=1
	
	#http://www.megaupload.com/?d=AGI7OZ4M
	#========================================================================
	def downloadFile(self, MUSrc, dest, row, continuedl=0):
		error = False
		fileDlEnd = False
		self.row = row
		while not fileDlEnd:
			# lancement du téléchargement
			try:
				MUSrc = str(MUSrc)
				dest = str(dest)
			except OSError, e:
				error = True
				messagefin = u"Erreur OS: function DownloadFile: %s" % e
				logError("download - downloadFile - 1", sys.exc_info()[0], sys.exc_info()[1])
			except:
				pass
			if not error:
				page = 0
				try:
					self.filename = dest
					self.msgInf = [u"Contact de %s..." % (MUSrc), 1]
					self.infotelech(0, 0, 0)
					page = self.httpreq.getPage(MUSrc)
					if not error:
						html = page.read()
						
						link = html.split("downloadlink")
						if len(link)>1:
							link = link[1].split('href="')[1]
							fileSrc = link.split('"')[0]
							sleepTime = 50
							if(dest=="auto"):
								dest = htmlentities_decode(fileSrc.split("/")[-1])
								
							self.filename = dest
							dest = os.path.join(self.destination, dest)
			
							while sleepTime > 0:
								self.msgInf = [u"Téléchargement de %s dans %ss" % (self.filename, sleepTime), 1]
								self.infotelech(0, 0, 0)
								sleepTime = sleepTime-1
								time.sleep(1)
							self.msgInf = [u"Téléchargement de %s en cours" % (self.filename), 2]
							self.fileRunning = MUSrc
							
							self.httpreq.defineContinue(continuedl)
							page = self.httpreq.getPage(fileSrc, dest=dest, report_hook=self.infotelech)
							self.httpreq.defineContinue(0)
						
							self.fileRunning = ''
							if isInt(page):
								if page == -1:
									messagefin = u"Téléchargement stoppé"
								elif page == -2:
									messagefin = u"l'Url est invalide ou pas de connexion internet 1"
								else:
									messagefin = u"Autre: Erreur %s" % page
							elif self.stop == True:
								messagefin = u"Téléchargement stoppé"
							else:
								messagefin = u"Téléchargement terminé\n\n"
						else:
							messagefin = u"Le lien indiqué n'est pas un lien MegaUpload valide"
				except Abort:
					messagefin = u"Téléchargement stoppé"
				except:
					messagefin = u"l'Url est invalide ou pas de connexion internet 2"
						
			# fin du téléchargement: émission du message de fin
			if(page==503):
				try:
					self.msgInf = [u"Download limit exceeded...", 1]
					self.infotelech(0, 0, 0)
					page = self.httpreq.getPage("http://www.megaupload.com/?c=premium&l=1")
					html = page.read()
					if("Please wait" in html):
						waiting = int(html.split("wait")[1].split("minutes")[0].replace(" ", ""))*60
						msg = u"Download limit exceeded, reprise du téléchargement dans"
					elif("currently downloading" in html):
						waiting = 5*60
						msg = u"Download limit exceeded, Votre IP télécharge déjà un fichier\nNouvelle tentative dans"
					else:
						try:
							logHtml = os.path.join(self.parent.configDir, "log.html")
							fp = open(logHtml, "w+")
							fp.write(html)
							fp.close()
						except:
							logError("download - downloadFile - 3", sys.exc_info()[0], sys.exc_info()[1])
						waiting = 120
						msg = "Error, new page in log.html"
					while waiting > 0 and not self.stop:
						time.sleep(1)
						
						if(waiting >= 60):
							timeSec = waiting%60
							timeMin = int(waiting/60)
							limitLeft = "%sm%ss" % (timeMin, timeSec)
						else:
							limitLeft = "%ss" (waiting)
						
						self.msgInf = [u"%s %s" % (msg, limitLeft), 1]
						self.infotelech(0, 0, 0)
						
						waiting = waiting-1
					fileDlEnd = False
				except Abort:
					pass
				except:
					logError("download - downloadFile - 4", sys.exc_info()[0], sys.exc_info()[1])
					fileDlEnd = True
			else:
				fileDlEnd = True
		if self.parent.appMethod == "gui":
			self.emit(SIGNAL("fintelech(PyQt_PyObject)"), [messagefin, row])
		else:
			self.parent.fintelech([messagefin, row])
		

	#========================================================================
	def infotelech(self, telechbloc, taillebloc, totalblocs):
		"""reçoit les infos de progression du téléchargement"""
		# nécessaire pour stopper le téléchargement avant la fin
		if self.stop:
			raise Abort
		# envoie les infos de progression à la fenêtre graphique
		if self.parent.appMethod == "gui":
			self.emit(SIGNAL("infotelech(PyQt_PyObject)"), [telechbloc, taillebloc, totalblocs, self.msgInf, self.row, self.filename])
		else:
			self.parent.infotelech([telechbloc, taillebloc, totalblocs, self.msgInf, self.row, self.filename])
		
 
	#========================================================================
	def stoptelech(self):
		# permet l'arrêt du téléchargement avant la fin
		self.stop = True
		
	#========================================================================
	def pausetelech(self):
		# permet la pause du téléchargement
		if self.httpreq.pause == True:
			self.httpreq.pause = False
		else:
			self.httpreq.pause = True
